@@ -37,7 +37,7 @@
 #include <QDebug>
 #include <QFile>
 #include <QMessageBox>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QStack>
 
 #include <functional>
@@ -126,10 +126,6 @@ QmlProfilerModelManager::QmlProfilerModelManager(QObject *parent) :
             this, &QmlProfilerModelManager::setTypeDetails);
     connect(d->detailsRewriter, &Internal::QmlProfilerDetailsRewriter::eventDetailsChanged,
             this, &QmlProfilerModelManager::typeDetailsFinished);
-
-    quint64 allFeatures = 0;
-    for (quint8 i = 0; i <= MaximumProfileFeature; ++i)
-        allFeatures |= (1ull << i);
 }
 
 QmlProfilerModelManager::~QmlProfilerModelManager()
@@ -246,10 +242,10 @@ static QString getInitialDetails(const QmlEventType &event)
             if (event.rangeType() == Javascript)
                 details = QmlProfilerModelManager::tr("anonymous function");
         } else {
-            QRegExp rewrite(QLatin1String("\\(function \\$(\\w+)\\(\\) \\{ (return |)(.+) \\}\\)"));
-            bool match = rewrite.exactMatch(details);
-            if (match)
-                details = rewrite.cap(1) + QLatin1String(": ") + rewrite.cap(3);
+            QRegularExpression rewrite(QLatin1String("^\\(function \\$(\\w+)\\(\\) \\{ (return |)(.+) \\}\\)$"));
+            QRegularExpressionMatch match = rewrite.match(details);
+            if (match.hasMatch())
+                details = match.captured(1) + QLatin1String(": ") + match.captured(3);
             if (details.startsWith(QLatin1String("file://")) ||
                     details.startsWith(QLatin1String("qrc:/")))
                 details = details.mid(details.lastIndexOf(QLatin1Char('/')) + 1);
@@ -399,7 +395,7 @@ QmlProfilerModelManager::rangeFilter(qint64 rangeStart, qint64 rangeEnd) const
                 if (type.rangeType() != MaximumRangeType) {
                     if (event.rangeStage() == RangeStart)
                         stack.push(event);
-                    else if (event.rangeStage() == RangeEnd)
+                    else if (event.rangeStage() == RangeEnd && !stack.isEmpty())
                         stack.pop();
                     return true;
                 } else if (isStateful(type)) {
